#! /usr/bin/perl

# HTMLSec.pl
# Version 1.0a
# Copyright 1996 by Richard Dawe
#
# Description
# ===========
#
#       HTMLSec updates sections in an HTML file. These are specified by
# start/end markers in the HTML file, which have the form of:
#
# <!-- *start <section name>* -->
# ...
# <!-- *end <section name>* -->
#
# where <section name> is the name of the section. The '...' between the
# markers is where HTMLSec will place the section. Any content there will be
# overwritten. NB: Case is important in naming these markers, as is the
# positioning - it must be the same as above.
#
#       The start & end markers must each be on one line - they cannot be on
# multiple lines, otherwise HTMLSec will not detect them. This is because of
# the simplistic detection method used.
#
# Syntax
# ======
#
# For syntax, see the help message below.

# Uses
use strict;             # Strict syntax
no strict 'vars';       # Except with variables, where it's annoying

# ***************************
# * Define global variables *
# ***************************

# Constants

# Help messages

# Copyright & version info
$help_program_info = <<EOT;
HTMLSec, Version 1.0a, Copyright 1996 by Richard Dawe
EOT

# E-mail address
$help_email_address = "rd5718\@irix.bris.ac.uk";

# Command-line help message
$help_message = <<EOT;
$help_program_info
Syntax: HTMLSec.pl [ <input file name>                                  ]
                   [ -d <section name>    || --delete <section name(s)> ]
                   [ -dbg                 || --debug                    ]
                   [ -h[h]                || --help[-html]              ]
                   [ -h[h]2               || --more-help[-html]         ]
                   [ -I <include dir.>    || --include <include dir.>   ]
                   [ -l                   || --list                     ]
                   [ -o <file name>       || --output <file name>       ]
                   [ -u <section name(s)> || --update <section name(s)> ]

        'Sections' are parts of HTML files that can be used to put common
features in HTML files (e.g. toolbars, copyright messages). Multiple sections
can be deleted/updated simultaneously by listing their names, or by using the
parameter(s) more than once.

        When looking for HTML file parts, HTMLSec will look first in the
current directory, then in the directory(s) specified with the '-I' option
and finally in the path(s) specified by the 'HTMLINCLUDE' environment
variable. The '-I' parameter can be used multiply.
EOT

# More detailed help message
$help_message_2 = <<EOT;
$help_program_info
        HTMLSec's various functions deal with sections. These are defined in
HTML files using the comment tag. The format of sections *must* be:

        <!-- *start <section name>* -->
        ...
        <!-- *end <section name>* -->

where <section name> specifies the section name. Note that there should be
exactly 1 space before the 1st asterisk, and 1 after it. HTMLSec will not be
able to discern the sections otherwise.

        When doing update operation, HTMLSec will destroy any text between the
two section markers. If no option is being performed on a section, then its
contents will be preserved. You cannot perform multiple operations on one
section - this would have to be performed in multiple passes (there is nothing
currently to be gained by performing multiple operations).
EOT

# HTML version of the help messages (for CGI scripts, automatic generation
# of HTML help)
$help_message_html = $help_message;
$help_message_2_html = $help_message_2;

# Replace <, &, and > in help messages with HTML escape sequences so they
# appear OK
$help_message_html =~ s/$help_program_info//;
$help_message_2_html =~ s/$help_program_info//;
$help_message_html =~ s/&/&amp;/g;
$help_message_2_html =~ s/&/&amp;/g;
$help_message_html =~ s/</&lt;/g;
$help_message_2_html =~ s/</&lt;/g;
$help_message_html =~ s/>/&gt;/g;
$help_message_2_html =~ s/>/&gt;/g;

# Syntax help message in HTML
$help_message_html = <<EOT;
<HEAD>
<TITLE>$help_program_info</TITLE>
</HEAD>
<BODY BGCOLOR="#FFFFFF">
<H1>$help_program_info</H1>
<PRE>$help_message_html</PRE><P>
<HR>
<I>This page was produced using HTMLSec's </I>'<TT>--help-html</TT>'<I>
option. For questions related to HTMLSec, please e-mail the author on
<A HREF="$help_email_address">$help_email_address</A>.</I>
</BODY>
</HTML>
EOT

# More detailed  help message in HTML
$help_message_2_html = <<EOT;
<HEAD>
<TITLE>$help_program_info</TITLE>
</HEAD>
<BODY BGCOLOR="#FFFFFF">
<H1>$help_program_info</H1>
<PRE>$help_message_2_html</PRE><P>
<HR>
<I>This page was produced using HTMLSec's </I>'<TT>--more-help-html</TT>'<I>
option. For questions related to HTMLSec, please e-mail the author on
<A HREF="$help_email_address">$help_email_address</A>.</I>
</BODY>
</HTML>
EOT

# Constants

# Section IDs
$SECTION_NONE = 0;
$SECTION_DELETE = 1;
$SECTION_UPDATE = 2;

# File IDs
$FILE_OUTPUT = -1;
$FILE_INCLUDE_PATH = -2;

# Help IDs
$HELP_BASIC_NORMAL = 1;
$HELP_BASIC_HTML = 2;
$HELP_MORE_NORMAL = 3;
$HELP_MORE_HTML = 4;

# File names to use
$file_in = "";                  # Input file
# FILE_IN                         File handle for input file
$file_out = "";                 # Output file
# FILE_OUT                        File handle for output file
@include_path = ();             # Include paths

# Options

# Section modifiers
$opt_delete = 0;                # Delete section
@opt_delete_list = ('');        # List of sections to delete

$opt_update = 0;                # Update section
@opt_update_list = ('');        # List of sections to update

# Other options
$opt_debug = 0;                 # Debugging information
$opt_help = 0;                  # Help information
$opt_list = 0;                  # List sections in file
$opt_output = 0;                # Output file specification

# Storage for HTML page
@html = ();

# ****************
# * Main program *
# ****************

# Check args
if ($#ARGV < 0)
{
        print $help_message;
        exit (0);
}

# Args OK, process 'em
&Parameters_Process;

# Any non-section actions to perform?
SWITCH:
{
        if ($opt_help == $HELP_BASIC_NORMAL) { print $help_message; exit (0); }
        if($opt_help == $HELP_BASIC_HTML) { print $help_message_html; exit (0); }
        if ($opt_help == $HELP_MORE_NORMAL) { print $help_message_2; exit (0); }
        if($opt_help == $HELP_MORE_HTML) { print $help_message_2_html; exit (0); }
}

# Get input file
&FileInProcess;

# List sections
if ($opt_list == 1) { &SectionsList; exit (0); }

# And write out new version
&FileOutProcess;

# Let's exit now everything's done
exit (0);

# ***************
# * Subroutines *
# ***************

# ----------------------
# - Parameters_Process -
# ----------------------

sub Parameters_Process
{
        # Get command-line arguments
        local $inNameSpec = 0;  # Section IDs used when previous parameter
                                # a section parameter; -1 when output file
                                # name

        # Include path temp var for environment string splitting, plus a
        # loop variable
        local (@include_path_temp, $i);

        for ($i = 0; $i <= $#ARGV; $i++)
        {
                # File name specification - process it
                if ($inNameSpec != 0)
                {
                        # Check 1st character != '-' - if so, then name
                        # specification is over. If the parameter is one
                        # which allows multiple file or section names, then
                        # continue search for file specs.

                        if (index ($ARGV[$i], '-') != 0)
                        {
                                # Name of section to delete
                                if ($inNameSpec == $SECTION_DELETE)
                                {                                      
                                        $opt_delete_list[$#opt_delete_list] = $ARGV[$i];
                                        $#opt_delete_list++;
                                }

                                # Name of section to update
                                if ($inNameSpec == $SECTION_UPDATE)
                                {                                        
                                        $opt_update_list[$#opt_update_list] = $ARGV[$i];
                                        $#opt_update_list++;
                                }

                                # Output file name
                                if ($inNameSpec == $FILE_OUTPUT)
                                {
                                        $file_out = $ARGV[$i];
                                        $opt_output = 1;
                                        $inNameSpec = 0;
                                }

                                # Include file path
                                if ($inNameSpec == $FILE_INCLUDE_PATH)
                                {
                                        $#include_path++;
                                        $include_path[$#include_path] = $ARGV[$i];
                                        $inNameSpec = 0;
                                }
                        }

                        # Found a parameter!
                        else { $inNameSpec = 0; }
                }

                # Normal parameter - check to see if test above fell through
                if ($inNameSpec == 0)
                {
                        # Debugging option
                        if (($ARGV[$i] eq '-dbg') || 
                                ($ARGV[$i] eq '--debug'))
                        {
                                # Turn on debugging
                                $opt_debug = 1;
                        }

                        # Help option
                        if (($ARGV[$i] eq '-h') ||
                                ($ARGV[$i] eq '--help'))
                        {
                                # Turn on help
                                $opt_help = $HELP_BASIC_NORMAL;
                        }

                        # Help in HTML
                        if (($ARGV[$i] eq '-hh') ||
                                ($ARGV[$i] eq '--help-html'))
                        {
                                # Turn on help
                                $opt_help = $HELP_BASIC_HTML;
                        }

                        # More help normally
                        if (($ARGV[$i] eq '-h2') ||
                                ($ARGV[$i] eq '--more-help'))
                        {
                                # Turn on help
                                $opt_help = $HELP_MORE_NORMAL;
                        }

                        # More help in HTML
                        if (($ARGV[$i] eq '-hh2') ||
                                ($ARGV[$i] eq '--more-help-html'))
                        {
                                # Turn on help
                                $opt_help = $HELP_MORE_HTML;
                        }

                        # List option
                        if (($ARGV[$i] eq '-l') ||
                                ($ARGV[$i] eq '--list'))
                        {
                                # List sections
                                $opt_list = 1;
                        }

                        # Output file name
                        if (($ARGV[$i] eq '-o') ||
                                ($ARGV[$i] eq '--output'))
                        {
                                # Get output file name
                                if ($file_out eq "")
                                { $inNameSpec = $FILE_OUTPUT; }
                        }

                        # Include file path
                        if (($ARGV[$i] eq '-I') ||
                                ($ARGV[$i] eq '--include'))
                        {
                                # Get output file name
                                $inNameSpec = $FILE_INCLUDE_PATH;
                        }

                        # Delete section
                        if (($ARGV[$i] eq '-d') ||
                                ($ARGV[$i] eq '--delete'))
                        {
                                $opt_delete = 1;

                                # Get output file name
                                $inNameSpec = $SECTION_DELETE;
                        }

                        # Update section
                        if (($ARGV[$i] eq '-u') ||
                                ($ARGV[$i] eq '--update'))
                        {
                                $opt_update = 1;

                                # Get output file name
                                $inNameSpec = $SECTION_UPDATE;
                        }

                        # Check that the parameter isn't a file name
                        if ((index ($ARGV[$i], '-') != 0) &&
                                ($file_in eq ""))
                        { $file_in = $ARGV[$i]; }
                }
        }

        # Get environment variable for include path - use DOS ; separator
        # Then copy it into existing search path array
        @include_path_temp = split (/\;/, $ENV{'HTMLINCLUDE'});

        for ($i = 0; $i <= $#include_path_temp; $i++)
        {
                $#include_path++;
                $include_path[$#include_path] = $include_path_temp[$i];
        }

        # Display 'em (debugging)
        if ($opt_debug == 1)
        {
                print STDERR $help_program_info . "\n";

                print STDERR "DEBUG: Command-line arguments : ";
                for ($i = 0; $i <= $#ARGV; $i++)
                {
                        print STDERR "\"" . $ARGV[$i]."\" ";
                }
                print STDERR "\n";

                print STDERR "DEBUG: Input file name        : $file_in\n";
                print STDERR "DEBUG: Output file name       : $file_out\n";

                print STDERR "DEBUG: Include search path    : ";
                for ($i = 0; $i <= $#include_path; $i++)
                { print "$include_path[$i] "; }
                print STDERR "\n";

                print STDERR "DEBUG: Sections to delete     : ";
                for ($i = 0; $i <= $#opt_delete_list; $i++)
                {
                        print STDERR $opt_delete_list[$i]." ";
                }
                print STDERR "\n";

                print STDERR "DEBUG: Sections to update     : ";
                for ($i = 0; $i <= $#opt_update_list; $i++)
                {
                        print STDERR $opt_update_list[$i]." ";
                }
                print STDERR "\n";                
        }
}

# -----------------
# - FileInProcess -
# -----------------

sub FileInProcess
{
        if (open (FILE_IN, "<$file_in") == 0) { die "Error: Unable to open input file '$file_in': $!"; }

        # Read in the file - store in html array
        while (<FILE_IN>)
        {                
                $#html++;
                $html[$#html] = $_;
        }

        close (FILE_IN);
}

# ------------------
# - FileOutProcess -
# ------------------

sub FileOutProcess
{
        # Temp vars - e.g. loops
        local ($i, $j, $k, @temp, @temp2);          

        # Section vars:
        #
        # - $section = section name
        # - $in_section = status flag - are we in a section in the HTML file?
        # - $section_file = file name of section to include - HTMLSec searches
        #       for the file in the include paths
        # - SECTIONFILE = file handle to $section_file

        local ($section, $in_section, $section_file);

        # Init vars
        $in_section = 0;

        # Open output file, if necessary
        if ($opt_output == 1)
        {
                if (open (FILE_OUT, ">$file_out") == 0) { die "Error: Unable to open output file '$file_out': $!"; }
                select (FILE_OUT);
        }

        # Output file
        for ($i = 0; $i <= $#html; $i++)
        {
                # Operate on $_ as if it is a file being read
                $_ = $html[$i];

                # Check if in a section or not - if not, find one
                if ($in_section == 0)
                {
                        # Print output to file
                        print;
                        
                        # Check to see if there is a section present
                        if (m/\<\!\-\- \*start / && m/\* \-\->/)
                        {
                                # Split line & retrieve section name
                                @temp = split (/\*/);
                                @temp2 = split (/ /, $temp[1]);
                                $section = $temp2[1];                                

                                # Put section in file, if update option spec'd

                                # Find if section spec'd for update
                                if ($opt_update == 1)
                                {
                                        for ($j = 0; $j <= $#opt_update_list; $j++)
                                        {
                                                if ($opt_update_list[$j] eq $section)
                                                {
                                                        # Get full file name 
                                                        # of section file &
                                                        # open it
                                                        $section_file = &SectionFindFile ($section);
                                                        if ($section_file eq "") { die "Unable to find section file '$section_file'!\n"; }
                                                        if (!open (SECTIONFILE, "<$section_file")) { die "Unable to open section file '$section_file': $!"; }

                                                        # Print to output file
                                                        while (<SECTIONFILE>)
                                                        { print; }

                                                        # Close section file
                                                        close (SECTIONFILE);

                                                        # Break out of loop
                                                        $in_section = 1;                                        $in_section = 1;
                                                        last;
                                                }

                                                # Error!
                                                if ($j == $#opt_update_list)
                                                { print STDERR "Error: Invalid section name: '$section'\n"; }
                                        }
                                }

                                # Find if section spec'd for deletion
                                if ($opt_delete == 1)
                                {
                                        # Find if the section is to be deleted
                                        # If it is, simply don't print it to
                                        # output file
                                        for ($j = 0; $j <= $#opt_delete_list; $j++)
                                        { if ($opt_delete_list[$j] eq $section) { $in_section = 1; last; } }
                                }

                                # No updating - leave as it is then
                                # $in_section = 0;
                        }
                }

                # Yep, in a section
                else                
                {
                        if (m/\<\!\-\- \*end $section\* \-\->/)
                        {
                                # OK - section over - carry on printing rest
                                # of file now section has been updated                                
                                print;
                                $in_section = 0;
                        }
                }
        }

        # Close output file
        if ($opt_output == 1)
        {
                select (STDOUT);
                close (FILE_OUT);
        }
}

# -------------------
# - SectionFindFile -
# -------------------
#
# Description: Looks for the spec'd section file in the search path, and
#       returns the fully qualified file name if found, or "" if not.

sub SectionFindFile
{
        local ($section) = @_;
        local ($i, $temp_file, $result);
        # DIRHANDLE = directory handle for search

        # No result
        $result = "";

        for ($i = 0; $i <= $#include_path; $i++)
        {
                $temp_file = opendir (DIRHANDLE, "$include_path[$i]");

                while ($temp_file ne "")
                {
                        # Found it! Return it!
                        if ($temp_file eq "$section.ihtml")
                        {
                                $result = $include_path[$i];

                                # Check for trailing slash
                                if (chop ($result) ne "/")
                                { $result = $include_path[$i]."/"; }

                                # Append section file name
                                $result = $result."$section.ihtml";

                                # Return
                                last;
                        }

                        # Get next dir entry
                        $temp_file = readdir (DIRHANDLE);
                }

                # Close directory
                close (DIRHANDLE);
        }

        # Return $result
        return ($result);
}

# ----------------
# - SectionsList -
# ----------------

sub SectionsList
{
        # Temp vars
        local (@temp, @temp2, $j);

        # Section list
        local $section;

        print "$help_program_info\n'$file_in' contains the following sections:\n";

        # Find HTML sections
        for ($j = 0; $j < $#html; $j++)
        {
                $_ = $html[$j];

                if (m/\<\!\-\- \*start / && m/\-\-\>/)
                {
                        @temp = split (/\*/);
                        @temp2 = split (/ /, $temp[1]);
                        $section = $temp2[1];                                
                        print "- $section\n";
                }
        }
}
